<?php

class NovinpalCF
{
    protected $apiKey;
    const NOVINPAL_BASE_URL_CF = 'https://api.novinpal.ir/invoice/';
    const NOVINPAL_REQUEST_URL_CF = self::NOVINPAL_BASE_URL_CF . 'request';
    const NOVINPAL_VERIFY_URL_CF = self::NOVINPAL_BASE_URL_CF . 'verify';
    const NOVINPAL_START_URL_CF = self::NOVINPAL_BASE_URL_CF . 'start/';
    const NOVINPAL_STATUS_VERIFIED_BEFORE_CF = 2;
    const NOVINPAL_STATUS_SUCCESS_CF = 1;
    const NOVINPAL_STATUS_ERROR_CF = -1;
    const NOVINPAL_STATUS_EMPTY_CF = -2;

    public function __construct($apiKey)
    {
        $this->apiKey = $apiKey;
    }

    public function request($amount, $callbackUrl, $orderId, $description = null, $mobile = null)
    {

        $data = [
            'api_key'    => $this->apiKey,
            'amount'     => $amount,
            'return_url' => $callbackUrl,
            'order_id'   => $orderId,
        ];

        if (!empty($description)) $data['description'] = $description;
        if (!empty($mobile)) $data['mobile'] = $mobile;

        $result = $this->postData(self::NOVINPAL_REQUEST_URL_CF, $data);
        $requestStatus = $this->getRequestStatus($result);

        return $this->getResponse($requestStatus, $result);
    }

    public function start($refId, $javascript = false)
    {

        $url = self::NOVINPAL_START_URL_CF . $refId;

        if ($javascript) {

            echo "<script>document.location = '${url}'</script><center>در صورتی که به صورت خودکار به درگاه بانک منتقل نشدید <a href='${url}'>اینجا</a> را کلیک کنید.</center>";
            return;
        }

        header('Location: ' . $url);
    }

    public function verify($refId, $amount)
    {

        $data = [
            'api_key' => $this->apiKey,
            'ref_id'  => $refId,
            'amount'  => $amount,
        ];

        $result =  $this->postData(self::NOVINPAL_VERIFY_URL_CF, $data);
        $verifyStatus = $this->getVerifyStatus($result);

        return $this->getResponse($verifyStatus, $result);
    }

    protected function postData($url, $data)
    {

        $query = http_build_query($data);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $query);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            'Accept: application/json',
            'Content-Type: application/x-www-form-urlencoded'
        ));
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        
        try {

            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_USERAGENT, 'NovinPal Rest Api v1');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

            $result = curl_exec($curl);
            curl_close($curl);

            $result = json_decode($result, true);
        } catch (Exception $exception) {

            $result = [];
        }

        if (!is_array($result))
            return [];

        return $result;
    }

    protected function getRequestStatus ($response) {

        if (empty($response)) {

            return self::NOVINPAL_STATUS_EMPTY_CF;
        }

        if (isset($response['status']) && $response['status'] == 1 && !empty($response['refId'])) {

            return self::NOVINPAL_STATUS_SUCCESS_CF;
        }

        return self::NOVINPAL_STATUS_ERROR_CF;
    }

    protected function getVerifyStatus ($response) {

        if (empty($response)) {

            return self::NOVINPAL_STATUS_EMPTY_CF;
        }

        if (isset($response['verifiedBefore']) && $response['verifiedBefore']) {

            return self::NOVINPAL_STATUS_VERIFIED_BEFORE_CF;
        }

        if (isset($response['status']) && $response['status'] == 1) {

            return self::NOVINPAL_STATUS_SUCCESS_CF;
        }

        return self::NOVINPAL_STATUS_ERROR_CF;
    }

    protected function getResponse ($status, $data) {

        return [
            'status' => $status,
            'data' => $data
        ];
    }
}